Class: ChefDK::PolicyfileCompiler
- Inherits:
-
Object
- Object
- ChefDK::PolicyfileCompiler
- Extended by:
- Forwardable
- Defined in:
- lib/chef-dk/policyfile_compiler.rb
Constant Summary collapse
- DEFAULT_DEMAND_CONSTRAINT =
">= 0.0.0".freeze
- SOURCE_TYPES_WITH_FIXED_VERSIONS =
Cookbooks from these sources lock that cookbook to exactly one version
[:git, :path].freeze
Instance Attribute Summary collapse
-
#dsl ⇒ Object
readonly
Returns the value of attribute dsl.
-
#install_report ⇒ Object
readonly
Returns the value of attribute install_report.
-
#storage_config ⇒ Object
readonly
Returns the value of attribute storage_config.
Class Method Summary collapse
Instance Method Summary collapse
- #all_cookbook_location_specs ⇒ Object
- #artifacts_graph ⇒ Object
- #build {|@dsl| ... } ⇒ Object
- #combined_run_lists ⇒ Object
- #combined_run_lists_by_cb_name ⇒ Object
- #cookbook_location_spec_for(cookbook_name) ⇒ Object
- #cookbook_version_fixed?(cookbook_name) ⇒ Boolean
- #cookbooks_in_run_list ⇒ Object
- #create_spec_for_cookbook(cookbook_name, version) ⇒ Object
- #default_attributes ⇒ Object
- #error! ⇒ Object
- #evaluate_policyfile(policyfile_string, policyfile_filename) ⇒ Object
- #expanded_named_run_lists ⇒ Object
- #expanded_run_list ⇒ Object
- #fixed_version_cookbooks_specs ⇒ Object
- #graph ⇒ Object
- #graph_demands ⇒ Object
-
#graph_solution ⇒ Object
Compilation Methods.
-
#initialize(ui: nil, chef_config: nil) ⇒ PolicyfileCompiler
constructor
A new instance of PolicyfileCompiler.
- #install ⇒ Object
-
#local_artifacts_graph ⇒ Object
Gives a dependency graph for cookbooks that are source from an alternate location.
- #lock ⇒ Object
- #normalized_named_run_lists ⇒ Object
-
#normalized_run_list ⇒ Object
copy of the expanded_run_list, properly formatted for use in a lockfile.
- #override_attributes ⇒ Object
- #remote_artifacts_graph ⇒ Object
- #solution_dependencies ⇒ Object
- #version_constraint_for(cookbook_name) ⇒ Object
Constructor Details
#initialize(ui: nil, chef_config: nil) ⇒ PolicyfileCompiler
Returns a new instance of PolicyfileCompiler.
59 60 61 62 63 64 65 66 67 68 |
# File 'lib/chef-dk/policyfile_compiler.rb', line 59 def initialize(ui: nil, chef_config: nil) @storage_config = Policyfile::StorageConfig.new @dsl = Policyfile::DSL.new(storage_config, chef_config: chef_config) @artifact_server_cookbook_location_specs = {} @merged_graph = nil @ui = ui || UI.null @install_report = Policyfile::Reports::Install.new(ui: @ui, policyfile_compiler: self) end |
Instance Attribute Details
#dsl ⇒ Object (readonly)
Returns the value of attribute dsl.
55 56 57 |
# File 'lib/chef-dk/policyfile_compiler.rb', line 55 def dsl @dsl end |
#install_report ⇒ Object (readonly)
Returns the value of attribute install_report.
57 58 59 |
# File 'lib/chef-dk/policyfile_compiler.rb', line 57 def install_report @install_report end |
#storage_config ⇒ Object (readonly)
Returns the value of attribute storage_config.
56 57 58 |
# File 'lib/chef-dk/policyfile_compiler.rb', line 56 def storage_config @storage_config end |
Class Method Details
.evaluate(policyfile_string, policyfile_filename, ui: nil, chef_config: nil) ⇒ Object
41 42 43 44 45 |
# File 'lib/chef-dk/policyfile_compiler.rb', line 41 def self.evaluate(policyfile_string, policyfile_filename, ui: nil, chef_config: nil) compiler = new(ui: ui, chef_config: chef_config) compiler.evaluate_policyfile(policyfile_string, policyfile_filename) compiler end |
Instance Method Details
#all_cookbook_location_specs ⇒ Object
164 165 166 167 168 169 170 171 |
# File 'lib/chef-dk/policyfile_compiler.rb', line 164 def all_cookbook_location_specs # in the installation process, we create "artifact_server_cookbook_location_specs" # for any cookbook that isn't sourced from a single-version source (e.g., # path and git only support one version at a time), but we might have # specs for them to track additional version constraint demands. Merging # in this order ensures the artifact_server_cookbook_location_specs "win". cookbook_location_specs.merge(@artifact_server_cookbook_location_specs) end |
#artifacts_graph ⇒ Object
223 224 225 |
# File 'lib/chef-dk/policyfile_compiler.rb', line 223 def artifacts_graph remote_artifacts_graph.merge(local_artifacts_graph) end |
#build {|@dsl| ... } ⇒ Object
299 300 301 302 |
# File 'lib/chef-dk/policyfile_compiler.rb', line 299 def build yield @dsl self end |
#combined_run_lists ⇒ Object
287 288 289 290 291 |
# File 'lib/chef-dk/policyfile_compiler.rb', line 287 def combined_run_lists .values.inject(.to_a) do |accum_run_lists, run_list| accum_run_lists |= run_list.to_a end end |
#combined_run_lists_by_cb_name ⇒ Object
293 294 295 296 297 |
# File 'lib/chef-dk/policyfile_compiler.rb', line 293 def combined_run_lists_by_cb_name combined_run_lists.inject({}) do |by_name_accum, run_list_item| by_name_accum end end |
#cookbook_location_spec_for(cookbook_name) ⇒ Object
76 77 78 |
# File 'lib/chef-dk/policyfile_compiler.rb', line 76 def cookbook_location_spec_for(cookbook_name) cookbook_location_specs[cookbook_name] end |
#cookbook_version_fixed?(cookbook_name) ⇒ Boolean
274 275 276 277 278 279 280 |
# File 'lib/chef-dk/policyfile_compiler.rb', line 274 def cookbook_version_fixed?(cookbook_name) if cookbook_location_spec = cookbook_location_spec_for(cookbook_name) cookbook_location_spec.version_fixed? else false end end |
#cookbooks_in_run_list ⇒ Object
282 283 284 285 |
# File 'lib/chef-dk/policyfile_compiler.rb', line 282 def cookbooks_in_run_list recipes = combined_run_lists.map { |recipe| recipe.name } recipes.map { |r| r[/^([^:]+)/, 1] } end |
#create_spec_for_cookbook(cookbook_name, version) ⇒ Object
157 158 159 160 161 162 |
# File 'lib/chef-dk/policyfile_compiler.rb', line 157 def create_spec_for_cookbook(cookbook_name, version) matching_source = best_source_for(cookbook_name) = matching_source.(cookbook_name, version) spec = Policyfile::CookbookLocationSpecification.new(cookbook_name, "= #{version}", , storage_config) @artifact_server_cookbook_location_specs[cookbook_name] = spec end |
#default_attributes ⇒ Object
104 105 106 |
# File 'lib/chef-dk/policyfile_compiler.rb', line 104 def default_attributes dsl.node_attributes.combined_default.to_hash end |
#error! ⇒ Object
70 71 72 73 74 |
# File 'lib/chef-dk/policyfile_compiler.rb', line 70 def error! unless errors.empty? raise PolicyfileError, errors.join("\n") end end |
#evaluate_policyfile(policyfile_string, policyfile_filename) ⇒ Object
304 305 306 307 308 |
# File 'lib/chef-dk/policyfile_compiler.rb', line 304 def evaluate_policyfile(policyfile_string, policyfile_filename) storage_config.use_policyfile(policyfile_filename) @dsl.eval_policyfile(policyfile_string) self end |
#expanded_named_run_lists ⇒ Object
90 91 92 93 94 95 |
# File 'lib/chef-dk/policyfile_compiler.rb', line 90 def named_run_lists.inject({}) do |, (name, run_list_items)| [name] = Chef::RunList.new(*run_list_items) end end |
#expanded_run_list ⇒ Object
80 81 82 83 |
# File 'lib/chef-dk/policyfile_compiler.rb', line 80 def # doesn't support roles yet... Chef::RunList.new(*run_list) end |
#fixed_version_cookbooks_specs ⇒ Object
310 311 312 313 314 |
# File 'lib/chef-dk/policyfile_compiler.rb', line 310 def fixed_version_cookbooks_specs @fixed_version_cookbooks_specs ||= cookbook_location_specs.select do |_cookbook_name, cookbook_location_spec| cookbook_location_spec.version_fixed? end end |
#graph ⇒ Object
183 184 185 186 187 188 189 190 191 192 193 194 |
# File 'lib/chef-dk/policyfile_compiler.rb', line 183 def graph @graph ||= Solve::Graph.new.tap do |g| artifacts_graph.each do |name, dependencies_by_version| dependencies_by_version.each do |version, dependencies| artifact = g.artifact(name, version) dependencies.each do |dep_name, constraint| artifact.dependency(dep_name, constraint) end end end end end |
#graph_demands ⇒ Object
210 211 212 213 214 215 216 217 218 219 220 221 |
# File 'lib/chef-dk/policyfile_compiler.rb', line 210 def graph_demands cookbooks_for_demands.map do |cookbook_name| spec = cookbook_location_spec_for(cookbook_name) if spec.nil? [ cookbook_name, DEFAULT_DEMAND_CONSTRAINT ] elsif spec.version_fixed? [ cookbook_name, "= #{spec.version}" ] else [ cookbook_name, spec.version_constraint.to_s ] end end end |
#graph_solution ⇒ Object
Compilation Methods
177 178 179 180 181 |
# File 'lib/chef-dk/policyfile_compiler.rb', line 177 def graph_solution return @solution if @solution cache_fixed_version_cookbooks @solution = Solve.it!(graph, graph_demands) end |
#install ⇒ Object
116 117 118 119 120 121 122 123 124 125 126 127 128 129 130 131 132 133 134 135 136 137 138 139 140 141 142 143 144 145 146 147 148 149 150 151 152 153 154 155 |
# File 'lib/chef-dk/policyfile_compiler.rb', line 116 def install ensure_cache_dir_exists cookbook_and_recipe_list = combined_run_lists.map(&:name).map do |recipe_spec| cookbook, _separator, recipe = recipe_spec.partition("::") recipe = "default" if recipe.empty? [cookbook, recipe] end missing_recipes_by_cb_spec = {} graph_solution.each do |cookbook_name, version| spec = cookbook_location_spec_for(cookbook_name) if spec.nil? || !spec.version_fixed? spec = create_spec_for_cookbook(cookbook_name, version) install_report.installing_cookbook(spec) spec.ensure_cached end required_recipes = cookbook_and_recipe_list.select { |cb_name, _recipe| cb_name == spec.name } missing_recipes = required_recipes.select { |_cb_name, recipe| !spec.cookbook_has_recipe?(recipe) } unless missing_recipes.empty? missing_recipes_by_cb_spec[spec] = missing_recipes end end unless missing_recipes_by_cb_spec.empty? = "The installed cookbooks do not contain all the recipes required by your run list(s):\n" missing_recipes_by_cb_spec.each do |spec, missing_items| << "#{spec}\nis missing the following required recipes:\n" missing_items.each { |_cb, recipe| << "* #{recipe}\n" } end << "\n" << "You may have specified an incorrect recipe in your run list,\nor this recipe may not be available in that version of the cookbook\n" raise CookbookDoesNotContainRequiredRecipe, end end |
#local_artifacts_graph ⇒ Object
Gives a dependency graph for cookbooks that are source from an alternate location. These cookbooks could have a different set of dependencies compared to an unmodified copy upstream. For example, the community site may have a cookbook “apache2” at version “1.10.4”, which the user has forked on github and modified the dependencies without changing the version number. To accomodate this, the local_artifacts_graph should be merged over the upstream’s artifacts graph.
234 235 236 237 238 239 240 241 |
# File 'lib/chef-dk/policyfile_compiler.rb', line 234 def local_artifacts_graph cookbook_location_specs.inject({}) do |local_artifacts, (cookbook_name, cookbook_location_spec)| if cookbook_location_spec.version_fixed? local_artifacts[cookbook_name] = { cookbook_location_spec.version => cookbook_location_spec.dependencies } end local_artifacts end end |
#lock ⇒ Object
112 113 114 |
# File 'lib/chef-dk/policyfile_compiler.rb', line 112 def lock @policyfile_lock ||= PolicyfileLock.build_from_compiler(self, storage_config) end |
#normalized_named_run_lists ⇒ Object
97 98 99 100 101 102 |
# File 'lib/chef-dk/policyfile_compiler.rb', line 97 def normalized_named_run_lists .inject({}) do |normalized, (name, run_list)| normalized[name] = run_list.map { |i| normalize_recipe(i) } normalized end end |
#normalized_run_list ⇒ Object
copy of the expanded_run_list, properly formatted for use in a lockfile
86 87 88 |
# File 'lib/chef-dk/policyfile_compiler.rb', line 86 def normalized_run_list .map { |i| normalize_recipe(i) } end |
#override_attributes ⇒ Object
108 109 110 |
# File 'lib/chef-dk/policyfile_compiler.rb', line 108 def override_attributes dsl.node_attributes.combined_override.to_hash end |
#remote_artifacts_graph ⇒ Object
243 244 245 246 247 248 249 250 251 252 253 254 255 256 257 258 259 260 261 262 263 |
# File 'lib/chef-dk/policyfile_compiler.rb', line 243 def remote_artifacts_graph @merged_graph ||= begin conflicting_cb_names = [] merged = {} default_source.each do |source| merged.merge!(source.universe_graph) do |conflicting_cb_name, _old, _new| if (preference = preferred_source_for_cookbook(conflicting_cb_name)) preference.universe_graph[conflicting_cb_name] elsif cookbook_could_appear_in_solution?(conflicting_cb_name) conflicting_cb_names << conflicting_cb_name {} # return empty set of versions else {} # return empty set of versions end end end handle_conflicting_cookbooks(conflicting_cb_names) merged end end |
#solution_dependencies ⇒ Object
196 197 198 199 200 201 202 203 204 205 206 207 208 |
# File 'lib/chef-dk/policyfile_compiler.rb', line 196 def solution_dependencies solution_deps = Policyfile::SolutionDependencies.new all_cookbook_location_specs.each do |name, spec| solution_deps.add_policyfile_dep(name, spec.version_constraint) end graph_solution.each do |name, version| transitive_deps = artifacts_graph[name][version] solution_deps.add_cookbook_dep(name, version, transitive_deps) end solution_deps end |
#version_constraint_for(cookbook_name) ⇒ Object
265 266 267 268 269 270 271 272 |
# File 'lib/chef-dk/policyfile_compiler.rb', line 265 def version_constraint_for(cookbook_name) if (cookbook_location_spec = cookbook_location_spec_for(cookbook_name)) && cookbook_location_spec.version_fixed? version = cookbook_location_spec.version "= #{version}" else DEFAULT_DEMAND_CONSTRAINT end end |